﻿using System;

using DevExpress.XtraBars;
using DevExpress.XtraEditors;
using DevExpress.XtraEditors.Repository;
using DevExpress.XtraGrid;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            Load                                     += Form_Load;
            this.clearPaymentBarButtonItem.ItemClick += clearPaymentBarButtonItem_ItemClick;
            this.setPaymentBarButtonItem.ItemClick   += setPaymentBarButtonItem_ItemClick;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 폼 로드시 처리하기 - Form_Load(sender, e)

        /// <summary>
        /// 폼 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_Load(object sender, EventArgs e)
        {
            RepositoryItemComboBox repositoryItemComboBox = new RepositoryItemComboBox();

            repositoryItemComboBox.Items.AddRange(DataHelper.CompanyNameArray);

            this.gridControl.RepositoryItems.Add(repositoryItemComboBox);

            this.gridControl.DataSource = DataHelper.GetRecordBindingList(10);

            this.gridView.Columns["CompanyName"].ColumnEdit              = repositoryItemComboBox;
            this.gridView.Columns["Image"      ].RealColumnEdit.NullText = "[load image]";

            FormatConditionRuleValue formatConditionRuleValue = new FormatConditionRuleValue();

            formatConditionRuleValue.PredefinedName = "Red Bold Text";
            formatConditionRuleValue.Condition      = FormatCondition.Greater;
            formatConditionRuleValue.Value1         = DateTime.Today;

            GridFormatRule gridFormatRule = new GridFormatRule();

            gridFormatRule.Rule       = formatConditionRuleValue;
            gridFormatRule.ApplyToRow = false;
            gridFormatRule.Column     = this.gridView.Columns["RequiredDate"];

            this.gridView.FormatRules.Add(gridFormatRule);

            this.gridView.BestFitColumns();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 지급액 클리어 바 버튼 항목 클릭시 처리하기 - clearPaymentBarButtonItem_ItemClick(sender, e)

        /// <summary>
        /// 지급액 클리어 바 버튼 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void clearPaymentBarButtonItem_ItemClick(object sender, ItemClickEventArgs e)
        {
            this.gridView.CloseEditor();

            Record record = this.gridView.GetFocusedRow() as Record;

            if(record == null)
            {
                return;
            }

            record.Payment = 0;
        }

        #endregion
        #region 지급액 설정 바 버튼 항목 클릭시 처리하기 - setPaymentBarButtonItem_ItemClick(sender, e)

        /// <summary>
        /// 지급액 설정 바 버튼 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void setPaymentBarButtonItem_ItemClick(object sender, ItemClickEventArgs e)
        {
            this.gridView.SetFocusedRowCellValue("Payment", 999);
        }

        #endregion
    }
}