﻿using System;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Drawing;
using System.Runtime.CompilerServices;

namespace TestProject
{
    /// <summary>
    /// 레코드
    /// </summary>
    public class Record : INotifyPropertyChanged
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 속성 변경시 - PropertyChanged

        /// <summary>
        /// 속성 변경시
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// ID
        /// </summary>
        private int id;

        /// <summary>
        /// 회사명
        /// </summary>
        private string companyName;

        /// <summary>
        /// 지급액
        /// </summary>
        private Nullable<decimal> payment;

        /// <summary>
        /// 요청일
        /// </summary>
        private DateTime requiredDate;

        /// <summary>
        /// 처리 여부
        /// </summary>
        private bool processed;

        /// <summary>
        /// 이미지
        /// </summary>
        private Image image;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region ID - ID

        /// <summary>
        /// ID
        /// </summary>
        public int ID
        {
            get
            {
                return this.id;
            }
            set
            {
                if(this.id != value)
                {
                    this.id = value;

                    FirePropertyChangedEvent();
                }
            }
        }

        #endregion
        #region 회사명 - CompanyName

        /// <summary>
        /// 회사명
        /// </summary>
        public string CompanyName
        {
            get
            {
                return this.companyName;
            }
            set
            {
                if(this.companyName != value)
                {
                    if(string.IsNullOrEmpty(value))
                    {
                        throw new Exception();
                    }

                    this.companyName = value;

                    FirePropertyChangedEvent();
                }
            }
        }

        #endregion
        #region 지급액 - Payment

        /// <summary>
        /// 지급액
        /// </summary>
        [DataType(DataType.Currency)]
        public Nullable<decimal> Payment
        {
            get
            {
                return this.payment;
            }
            set
            {
                if(this.payment != value)
                {
                    this.payment = value;

                    FirePropertyChangedEvent();
                }
            }
        }

        #endregion
        #region 요청 일자 - RequiredDate

        /// <summary>
        /// 요청 일자
        /// </summary>
        [DisplayFormat(DataFormatString = "d")]
        public DateTime RequiredDate
        {
            get
            {
                return this.requiredDate;
            }
            set
            {
                if(this.requiredDate != value)
                {
                    this.requiredDate = value;

                    FirePropertyChangedEvent();
                }
            }
        }

        #endregion
        #region 처리 여부 - Processed

        /// <summary>
        /// 처리 여부
        /// </summary>
        public bool Processed
        {
            get
            {
                return this.processed;
            }
            set
            {
                if(this.processed != value)
                {
                    this.processed = value;

                    FirePropertyChangedEvent();
                }
            }
        }

        #endregion
        #region 이미지 - Image

        /// <summary>
        /// 이미지
        /// </summary>
        public Image Image
        {
            get
            {
                return this.image;
            }
            set
            {
                if(this.image != value)
                {
                    this.image = value;

                    FirePropertyChangedEvent();
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - Record()

        /// <summary>
        /// 생성자
        /// </summary>
        public Record()
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 문자열 구하기 - ToString()

        /// <summary>
        /// 문자열 구하기
        /// </summary>
        /// <returns>문자열</returns>
        public override string ToString()
        {
            return string.Format("ID = {0}, CompanyName = {1}", ID, CompanyName);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 속성 변경시 이벤트 발생시키기 - FirePropertyChangedEvent(propertyName)

        /// <summary>
        /// 속성 변경시 이벤트 발생시키기
        /// </summary>
        /// <param name="propertyName">속성명</param>
        protected void FirePropertyChangedEvent([CallerMemberName] string propertyName = "")
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}